/* Generated automatically. */
static const char configuration_arguments[] = "/opt/conda/conda-bld/gcc-compiler_1652852770113/work/gcc/configure --build=powerpc64le-build_conda_cos7-linux-gnu --host=powerpc64le-build_conda_cos7-linux-gnu --target=powerpc64le-conda-linux-gnu --prefix=/opt/conda/conda-bld/gcc-compiler_1652852770113/work/gcc_built --exec_prefix=/opt/conda/conda-bld/gcc-compiler_1652852770113/work/gcc_built --with-sysroot=/opt/conda/conda-bld/gcc-compiler_1652852770113/work/gcc_built/powerpc64le-conda-linux-gnu/sysroot --with-local-prefix=/opt/conda/conda-bld/gcc-compiler_1652852770113/work/gcc_built/powerpc64le-conda-linux-gnu/sysroot --enable-long-long --disable-multilib --disable-nls --with-gmp=/opt/conda/conda-bld/gcc-compiler_1652852770113/work/buildtools --with-mpfr=/opt/conda/conda-bld/gcc-compiler_1652852770113/work/buildtools --with-mpc=/opt/conda/conda-bld/gcc-compiler_1652852770113/work/buildtools --with-isl=/opt/conda/conda-bld/gcc-compiler_1652852770113/work/buildtools --without-zstd --enable-languages=c,c++,fortran,objc,obj-c++ --enable-__cxa_atexit --disable-libmudflap --enable-libgomp --enable-libquadmath --enable-libquadmath-support --enable-libsanitizer --disable-libstdcxx-verbose --enable-lto --enable-libcc1 --enable-threads=posix --enable-plugin --with-pkgversion='Anaconda gcc' ";
static const char thread_model[] = "posix";

static const struct {
  const char *name, *value;
} configure_default_options[] = { { NULL, NULL} };
